@@ -76,18 +76,25 @@ impl<'x> Header for URL<'x> {
         mut bytes_written: usize,
     ) -> std::io::Result<usize> {
         for (pos, url) in self.url.iter().enumerate() {
+            if pos > 0 {
+                if bytes_written + url.len() + 2 >= 76 {
+                    output.write_all(b"\r\n\t")?;
+                    bytes_written = 1;
+                } else {
+                    output.write_all(b" ")?;
+                    bytes_written += 1;
+                }
+            }
             output.write_all(b"<")?;
             output.write_all(url.as_bytes())?;
             output.write_all(b">")?;
             bytes_written += url.len() + 2;
-            if bytes_written >= 76 && pos < self.url.len() - 1 {
-                output.write_all(b"\r\n\t")?;
-                bytes_written = 0;
-            }
         }
+
         if bytes_written > 0 {
             output.write_all(b"\r\n")?;
         }
+
         Ok(0)
     }
 }
