@@ -11,13 +11,11 @@
 
 use std::{
     borrow::Cow,
-    collections::{
-        hash_map::{DefaultHasher, Entry},
-        HashMap,
-    },
+    collections::{btree_map::Entry, hash_map::DefaultHasher, BTreeMap},
     hash::{Hash, Hasher},
     io::{self, Write},
     iter::FromIterator,
+    thread,
     time::{Duration, SystemTime, UNIX_EPOCH},
 };
 
@@ -36,7 +34,7 @@ use crate::{
 
 /// MIME part of an e-mail.
 pub struct MimePart<'x> {
-    pub headers: HashMap<Cow<'x, str>, HeaderType<'x>>,
+    pub headers: BTreeMap<Cow<'x, str>, HeaderType<'x>>,
     pub contents: BodyPart<'x>,
 }
 
@@ -73,6 +71,7 @@ impl<'x> From<Vec<u8>> for BodyPart<'x> {
 pub fn make_boundary() -> String {
     let mut s = DefaultHasher::new();
     gethostname::gethostname().hash(&mut s);
+    thread::current().id().hash(&mut s);
     format!(
         "{:x}_{:x}_{:x}",
         SystemTime::now()
@@ -89,7 +88,7 @@ impl<'x> MimePart<'x> {
     pub fn new(content_type: ContentType<'x>, contents: BodyPart<'x>) -> Self {
         Self {
             contents,
-            headers: HashMap::from_iter(vec![("Content-Type".into(), content_type.into())]),
+            headers: BTreeMap::from_iter(vec![("Content-Type".into(), content_type.into())]),
         }
     }
 
@@ -100,7 +99,7 @@ impl<'x> MimePart<'x> {
     ) -> Self {
         Self {
             contents: BodyPart::Multipart(contents),
-            headers: HashMap::from_iter(vec![(
+            headers: BTreeMap::from_iter(vec![(
                 "Content-Type".into(),
                 ContentType::new(content_type).into(),
             )]),
@@ -111,7 +110,7 @@ impl<'x> MimePart<'x> {
     pub fn new_text(contents: impl Into<Cow<'x, str>>) -> Self {
         Self {
             contents: BodyPart::Text(contents.into()),
-            headers: HashMap::from_iter(vec![(
+            headers: BTreeMap::from_iter(vec![(
                 "Content-Type".into(),
                 ContentType::new("text/plain")
                     .attribute("charset", "utf-8")
@@ -127,7 +126,7 @@ impl<'x> MimePart<'x> {
     ) -> Self {
         Self {
             contents: BodyPart::Text(contents.into()),
-            headers: HashMap::from_iter(vec![(
+            headers: BTreeMap::from_iter(vec![(
                 "Content-Type".into(),
                 ContentType::new(content_type)
                     .attribute("charset", "utf-8")
@@ -140,7 +139,7 @@ impl<'x> MimePart<'x> {
     pub fn new_html(contents: impl Into<Cow<'x, str>>) -> Self {
         Self {
             contents: BodyPart::Text(contents.into()),
-            headers: HashMap::from_iter(vec![(
+            headers: BTreeMap::from_iter(vec![(
                 "Content-Type".into(),
                 ContentType::new("text/html")
                     .attribute("charset", "utf-8")
@@ -153,7 +152,7 @@ impl<'x> MimePart<'x> {
     pub fn new_binary(c_type: impl Into<Cow<'x, str>>, contents: impl Into<Cow<'x, [u8]>>) -> Self {
         Self {
             contents: BodyPart::Binary(contents.into()),
-            headers: HashMap::from_iter(vec![(
+            headers: BTreeMap::from_iter(vec![(
                 "Content-Type".into(),
                 ContentType::new(c_type).into(),
             )]),
@@ -202,11 +201,22 @@ impl<'x> MimePart<'x> {
     }
 
     /// Set custom headers of a MIME part.
-    pub fn header(mut self, header: impl Into<Cow<'x, str>>, value: HeaderType<'x>) -> Self {
-        self.headers.insert(header.into(), value);
+    pub fn header(
+        mut self,
+        header: impl Into<Cow<'x, str>>,
+        value: impl Into<HeaderType<'x>>,
+    ) -> Self {
+        self.headers.insert(header.into(), value.into());
         self
     }
 
+    /// Add a body part to a multipart/* MIME part.
+    pub fn add_part(&mut self, part: MimePart<'x>) {
+        if let BodyPart::Multipart(ref mut parts) = self.contents {
+            parts.push(part);
+        }
+    }
+
     /// Write the MIME part to a writer.
     pub fn write_part(self, mut output: impl Write) -> io::Result<usize> {
         let mut stack = Vec::new();
@@ -214,7 +224,7 @@ impl<'x> MimePart<'x> {
         let mut boundary: Option<Cow<str>> = None;
 
         loop {
-            while let Some(part) = it.next() {
+            while let Some(mut part) = it.next() {
                 if let Some(boundary) = boundary.as_ref() {
                     output.write_all(b"\r\n--")?;
                     output.write_all(boundary.as_bytes())?;
@@ -222,57 +232,90 @@ impl<'x> MimePart<'x> {
                 }
                 match part.contents {
                     BodyPart::Text(text) => {
+                        let mut is_attachment = false;
                         for (header_name, header_value) in &part.headers {
                             output.write_all(header_name.as_bytes())?;
                             output.write_all(b": ")?;
+                            if !is_attachment && header_name == "Content-Disposition" {
+                                is_attachment = header_value
+                                    .as_content_type()
+                                    .map(|v| v.is_attachment())
+                                    .unwrap_or(false);
+                            }
                             header_value.write_header(&mut output, header_name.len() + 2)?;
                         }
-                        match get_encoding_type(text.as_ref(), false) {
-                            EncodingType::Base64 => {
-                                output.write_all(b"Content-Transfer-Encoding: base64\r\n\r\n")?;
-                                base64_encode(text.as_bytes(), &mut output, false)?;
-                            }
-                            EncodingType::QuotedPrintable(_) => {
-                                output.write_all(
-                                    b"Content-Transfer-Encoding: quoted-printable\r\n\r\n",
-                                )?;
-                                quoted_printable_encode(text.as_bytes(), &mut output, false)?;
-                            }
-                            EncodingType::None => {
-                                output.write_all(b"Content-Transfer-Encoding: 7bit\r\n\r\n")?;
-                                output.write_all(text.as_bytes())?;
-                            }
-                        }
+                        detect_encoding(text.as_bytes(), &mut output, !is_attachment)?;
                     }
                     BodyPart::Binary(binary) => {
+                        let mut is_text = false;
+                        let mut is_attachment = false;
                         for (header_name, header_value) in &part.headers {
                             output.write_all(header_name.as_bytes())?;
                             output.write_all(b": ")?;
+                            if !is_text && header_name == "Content-Type" {
+                                is_text = header_value
+                                    .as_content_type()
+                                    .map(|v| v.is_text())
+                                    .unwrap_or(false);
+                            } else if !is_attachment && header_name == "Content-Disposition" {
+                                is_attachment = header_value
+                                    .as_content_type()
+                                    .map(|v| v.is_attachment())
+                                    .unwrap_or(false);
+                            }
                             header_value.write_header(&mut output, header_name.len() + 2)?;
                         }
-                        output.write_all(b"Content-Transfer-Encoding: base64\r\n\r\n")?;
-                        base64_encode(binary.as_ref(), &mut output, false)?;
+                        if !is_text {
+                            output.write_all(b"Content-Transfer-Encoding: base64\r\n\r\n")?;
+                            base64_encode(binary.as_ref(), &mut output, false)?;
+                        } else {
+                            detect_encoding(binary.as_ref(), &mut output, !is_attachment)?;
+                        }
                     }
                     BodyPart::Multipart(parts) => {
                         if boundary.is_some() {
                             stack.push((it, boundary));
                         }
 
-                        boundary = Some(make_boundary().into());
-
-                        for (header_name, mut header_value) in part.headers {
-                            match &mut header_value {
-                                HeaderType::ContentType(ref mut content_type)
-                                    if header_name == "Content-Type" =>
-                                {
+                        output.write_all(b"Content-Type: ")?;
+                        boundary = if let Some(value) = part.headers.remove("Content-Type") {
+                            match value {
+                                HeaderType::ContentType(mut ct) => {
                                     if let Entry::Vacant(entry) =
-                                        content_type.attributes.entry("boundary".into())
+                                        ct.attributes.entry("boundary".into())
                                     {
-                                        entry.insert(boundary.as_ref().unwrap().as_ref().into());
+                                        entry.insert(make_boundary().into());
+                                    }
+                                    ct.write_header(&mut output, 14)?;
+                                    ct.attributes.remove("boundary")
+                                }
+                                HeaderType::Raw(raw) => {
+                                    if let Some(pos) = raw.raw.find("boundary=\"") {
+                                        if let Some(boundary) = raw.raw[pos..].split('"').nth(1) {
+                                            Some(boundary.to_string().into())
+                                        } else {
+                                            Some(make_boundary().into())
+                                        }
+                                    } else {
+                                        let boundary = make_boundary();
+                                        output.write_all(raw.raw.as_bytes())?;
+                                        output.write_all(b"; boundary=\"")?;
+                                        output.write_all(boundary.as_bytes())?;
+                                        output.write_all(b"\"\r\n")?;
+                                        Some(boundary.into())
                                     }
                                 }
-                                _ => {}
+                                _ => panic!("Unsupported Content-Type header value."),
                             }
+                        } else {
+                            let boundary = make_boundary();
+                            ContentType::new("multipart/mixed")
+                                .attribute("boundary", &boundary)
+                                .write_header(&mut output, 14)?;
+                            Some(boundary.into())
+                        };
+
+                        for (header_name, header_value) in part.headers {
                             output.write_all(header_name.as_bytes())?;
                             output.write_all(b": ")?;
                             header_value.write_header(&mut output, header_name.len() + 2)?;
@@ -297,3 +340,32 @@ impl<'x> MimePart<'x> {
         Ok(0)
     }
 }
+
+fn detect_encoding(input: &[u8], mut output: impl Write, is_body: bool) -> io::Result<()> {
+    match get_encoding_type(input, false, is_body) {
+        EncodingType::Base64 => {
+            output.write_all(b"Content-Transfer-Encoding: base64\r\n\r\n")?;
+            base64_encode(input, &mut output, false)?;
+        }
+        EncodingType::QuotedPrintable(_) => {
+            output.write_all(b"Content-Transfer-Encoding: quoted-printable\r\n\r\n")?;
+            quoted_printable_encode(input, &mut output, false, is_body)?;
+        }
+        EncodingType::None => {
+            output.write_all(b"Content-Transfer-Encoding: 7bit\r\n\r\n")?;
+            if is_body {
+                let mut prev_ch = 0;
+                for ch in input {
+                    if *ch == b'\n' && prev_ch != b'\r' {
+                        output.write_all(b"\r")?;
+                    }
+                    output.write_all(&[*ch])?;
+                    prev_ch = *ch;
+                }
+            } else {
+                output.write_all(input)?;
+            }
+        }
+    }
+    Ok(())
+}
